/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.perseus.cache;

import org.objectweb.perseus.cache.api.CacheManager;
import org.objectweb.perseus.cache.api.CacheAttributeController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.StringTokenizer;

/**
 * Manages an unique instance of a CacheManager.
 * (singleton pattern)
 *
 * @author S.Chassande-Barrioz
 */
public class CacheManagerFactory {


	private static CacheManager singleton = null;

	private final static String CACHE_MANAGER_TM =
		"org.objectweb.perseus.cache.lib.SimpleCacheManager";

	public static CacheManager getCacheManager() throws Exception {
		if (singleton == null) {
			synchronized(CacheManagerFactory.class) {
				if (singleton == null) {
					singleton = newCacheManager();
				}
			}
		}
		return singleton;
	}

	public static CacheManager newCacheManager() throws Exception {
		Component cm = FractalHelper.instanciate(CACHE_MANAGER_TM);
		Fractal.getLifeCycleController(cm).startFc();
		return (CacheManager) cm.getFcInterface("cache-manager");
	}

	public static CacheAttributeController getCacheAttributeController(CacheManager cm) throws Exception {
		Component primitivecm = getSubComponent(
			((Interface) cm).getFcItfOwner(), "cache-manager");
		if (primitivecm == null) {
			throw new Exception("Impossible to find the CacheAttributeController for the given CacheManager: " + cm);
		}
		return (CacheAttributeController) Fractal.getAttributeController(primitivecm);
	}

	private static Component getSubComponent(Component parent,
									  String path) throws Exception {
		StringTokenizer st = new StringTokenizer(path, ".", false);
		Component res = parent;
		while(st.hasMoreTokens()) {
			String commponenentname = st.nextToken();
			Component[] children = Fractal.getContentController(res)
					.getFcSubComponents();
			int i = 0;
			while(i<children.length &&
					!Fractal.getNameController(children[i])
						.getFcName().equals(commponenentname)) {
				i++;
			}
			if (i<children.length) {
				res = children[i];
			} else {
				return null;
			}
		}
		return res;
	}

}
